import React, { Fragment, useState, useEffect } from 'react';
import * as PropTypes from 'prop-types';
import 'date-fns';
import DateFnsUtils from '@date-io/date-fns';
import { MuiPickersUtilsProvider, KeyboardTimePicker, KeyboardDatePicker, KeyboardDateTimePicker } from '@material-ui/pickers';
import { makeStyles } from '@material-ui/core/styles';

import getStyles from './DateTimePicker.styles';

const DATE_TYPES = {
    DATE: 'date',
    DATETIME: 'datetime',
    TIME: 'time',
};
const DATE_VIEWS = {
    DATE: 'date',
    HOURS: 'hours',
    MINUTES: 'minutes',
    YEAR: 'year',
};

/**
 * DateTimePicker component for picking date and time from user
 * @see https://material-ui.com/demos/pickers/#material-ui-pickers
 * @see https://material-ui-pickers.dev/getting-started/usage
 */
const DateTimePicker = (props) => {
    const {
        autoFocus,
        contextStyles,
        disabled,
        emptyLabel,
        format,
        is24Hour,
        mode,
        onChange,
        selectedDate,
        shouldDisableDate,
        testID,
        views,
    } = props;

    const {
        container: containerStyles,
        ...muiStyles
    } = getStyles(contextStyles);
    const styles = makeStyles(muiStyles)();
    const [
        value,
        setDate,
    ] = useState(selectedDate);
    const [
        open,
        setOpen,
    ] = useState(autoFocus);

    // Update when state tracked props are updated
    useEffect(() => {
        setDate(selectedDate);
        setOpen(autoFocus);
    }, [
        autoFocus,
        selectedDate,
    ]);

    // Generate container props
    const containerProps = {
        className: 'c-date-time-picker__container',
        style: containerStyles,
    };
    if (testID) { containerProps['data-test-id'] = `${testID}__date-time-picker__container`; }

    // Generate picker props
    const pickerProps = {
        ampm: !is24Hour,
        classes: {
            root: styles.inputContainer,
        },
        InputProps: {
            classes: {
                root: styles.inputContainer,
                input: styles.inputText,
            },
        },
        KeyboardButtonProps: {
            classes: {
                root: styles.iconContainer,
                label: styles.icon,
            },
        },
        DialogProps: {
            classes: {
                dialogRoot: styles.dialogContainer,
            },
        },
        className: 'c-date-time-picker__picker',
        clearable: true,
        emptyLabel,
        okLabel: 'Set',
        onAccept: (date) => {
            setOpen(false);
            if (onChange) {
                onChange(date, props);
            }
        },
        onChange: (date) => {
            setDate(date);
        },
        onClose: () => {
            setOpen(false);
        },
        onOpen: () => {
            setOpen(true);
        },
        open,
        value,
        variant: 'dialog',
        views,
    };
    if (format) { pickerProps.format = format; }
    if (shouldDisableDate) { pickerProps.shouldDisableDate = shouldDisableDate; }
    if (disabled) { pickerProps.disabled = disabled; }
    if (testID) { pickerProps['data-test-id'] = `${testID}__date-time-picker__picker`; }

    return (
        <div { ...containerProps }>
            <MuiPickersUtilsProvider utils={ DateFnsUtils }>
                <Fragment>
                    { mode === DATE_TYPES.DATE &&
                        <KeyboardDatePicker { ...pickerProps } />
                    }
                    { mode === DATE_TYPES.TIME &&
                        <KeyboardTimePicker { ...pickerProps } />
                    }
                    { mode === DATE_TYPES.DATETIME &&
                        <KeyboardDateTimePicker { ...pickerProps } />
                    }
                </Fragment>
            </MuiPickersUtilsProvider>
        </div>
    );
};

DateTimePicker.propTypes = {
    /** Auto Focuses on the input */
    autoFocus: PropTypes.bool,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the wrapping container surrounding the input container */
        container: PropTypes.object,

        /** Styles for the container surrounding the input */
        inputContainer: PropTypes.object,

        /** Styles for the text input */
        inputText: PropTypes.object,

        /** Styles for the text input icon */
        icon: PropTypes.object,

        /** Styles for the container surrounding the text input icon */
        iconContainer: PropTypes.object,

        /** Picker modal */
        dialogContainer: PropTypes.object,

        /**
         * Central picker modal content area containing:
         * Year/Month/Day/Hour/Time/AmPm selection buttons
         * Date/Time tabs
         * Prev/Next month buttons
         * Month/Year label
         * Day of week labels
         * Month view of days
         */
        dialogContentContainer: PropTypes.object,

        /**
         * Topmost picker modal date selection container containing:
         * Year button
         * Month/day button
         * Hour/minute/ampm button
         */
        pickerToolbar: PropTypes.object,

        /** Topmost picker modal year/month/day/time selection buttons */
        pickerToolbarButton: PropTypes.object,

        /** Topmost picker modal year/month/day/time selection button labels */
        pickerToolbarButtonLabels: PropTypes.object,

        /** Topmost picker modal hour/minute separator (:) */
        pickerDateTimeToolbarSeparator: PropTypes.object,

        /**
         * Date/Time selection tabs between Picker Toolbar (year/month/day/time controls)
         * and main calendar control month view content and buttons
         */
        pickerTabs: PropTypes.object,

        /** Tab button */
        pickerTabButtons: PropTypes.object,

        /** Tab button selected indicator */
        pickerTabIndicator: PropTypes.object,

        /**
         * Central picker modal content area containing:
         * Prev/Next month buttons
         * Month/Year label
         * Day of week labels
         * Month view of days
         */
        pickerView: PropTypes.object,

        /** Calendar prev/next buttons and Month/Year label container */
        pickerCalendarSwitchHeader: PropTypes.object,

        /** Calendar prev/next buttons */
        pickerCalendarButton: PropTypes.object,

        /** Calendar prev/next button icon */
        pickerCalendarButtonIcon: PropTypes.object,

        /** Calendar Month/Year label container */
        pickerCalendarHeaderLabel: PropTypes.object,

        /** Calendar days of week container */
        pickerCalendarDaysHeader: PropTypes.object,

        /** Calendar days of week labels */
        pickerCalendarDayLabel: PropTypes.object,

        /** Calendar month view container below days of week container */
        pickerCalendarContainer: PropTypes.object,

        /** Calendar week row */
        pickerCalendarWeek: PropTypes.object,

        /** Calendar day */
        pickerDay: PropTypes.object,

        /** Calendar day current day state */
        pickerDayCurrent: PropTypes.object,

        /** Calendar day selected state */
        pickerDaySelected: PropTypes.object,

        /** Calendar day disabled state */
        pickerDayDisabled: PropTypes.object,

        /** Calendar hidden days */
        pickerDayHidden: PropTypes.object,

        /** Calendar year selection container */
        pickerYearContainer: PropTypes.object,

        /** Calendar year item container */
        pickerYear: PropTypes.object,

        /** Calendar year item container selected state */
        pickerYearSelected: PropTypes.object,

        /** Calendar year item container disabled state */
        pickerYearDisabled: PropTypes.object,

        /** Main container surrounding the clock */
        pickerClockContainer: PropTypes.object,

        /** Clock container */
        pickerClock: PropTypes.object,

        /** Clock central pin */
        pickerClockPin: PropTypes.object,

        /** Clock number */
        pickerClockNumber: PropTypes.object,

        /** Clock number selected state */
        pickerClockNumberSelected: PropTypes.object,

        /** Clock hand from center point to number */
        pickerClockPointer: PropTypes.object,

        /** Clock indicator center not on a number */
        pickerClockPointerThumb: PropTypes.object,

        /** Clock indicator center on a number */
        pickerClockPointerNoPoint: PropTypes.object,

        /**
         * Bottom picker modal action area below content area containing:
         * Cancel/Set buttons
         */
        dialogActionsContainer: PropTypes.object,

        /** Action button containers */
        dialogActionsButtons: PropTypes.object,

        /** Action button labels */
        dialogActionsLabels: PropTypes.object,
    }),

    /** Prevents the control from changing input */
    disabled: PropTypes.bool,

    /** Message displaying in text field, if null passed */
    emptyLabel: PropTypes.string,

    /** Custom display format (see NPM package date-fns) */
    format: PropTypes.string,

    /** Display the date picker in a 24 hour format */
    is24Hour: PropTypes.bool,

    /** Date type format: date, time, datetime */
    mode: PropTypes.oneOf([
        'date',
        'time',
        'datetime',
    ]),

    /**
     * Called when the picker date has changed
     * @param {Date} date - The selected date
     * @param {Object} props - The component instance props
     */
    onChange: PropTypes.func,

    /** Selected date value */
    selectedDate: PropTypes.instanceOf(Date),

    /**
     * Called for each day on the calendar view
     * @param {Date} the calendar day
     * @returns {Boolean} disables the calendar day
     */
    shouldDisableDate: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Visible picker views */
    views: PropTypes.arrayOf(PropTypes.string),
};

DateTimePicker.defaultProps = {
    contextStyles: {},
    format: 'M/dd/yy, h:mm a',
    mode: DATE_TYPES.DATETIME,
    autoFocus: false,
    selectedDate: null,
    views: [
        DATE_VIEWS.DATE,
        DATE_VIEWS.HOURS,
        DATE_VIEWS.MINUTES,
        DATE_VIEWS.YEAR,
    ],
};

export default DateTimePicker;
export {
    DATE_TYPES,
    DATE_VIEWS,
};
