import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import * as PropTypes from 'prop-types';
import { AgGridColumn, AgGridReact } from 'ag-grid-react';
import SortableHeaderComponent from './SortableHeaderComponent';
import FilterComponent from './FilterComponent';

// TODO: SCOPE!!!!!!!
// import './DataGrid.css';
// import 'ag-grid-community/dist/styles/ag-grid.css';
// import 'ag-grid-community/dist/styles/ag-theme-material.css';
// import 'bootstrap/dist/css/bootstrap.css';
// import '@fortawesome/fontawesome-free/css/all.css';

export default class DataGrid extends Component {
    constructor(props) {
        super(props);
        const {
            cols,
        } = props;

        // Store mapping of column and width
        const colWidths = new Map();
        cols.map((col) => {
            colWidths.set(col.field, col.width);
        });

        this.state = {
            theme: 'material',
            columnList: cols,
            colWidths,
            viewPortScrollLeft: 0,
        };
    }

    render() {
        const {
            columnList,
            theme,
        } = this.state;

        const {
            rows,
            testID,
            multiSelectMode,
            contextStyles,
        } = this.props;

        // Create and add stylesheet from contextStyles object dynamically to document.
        const rowSelectedCSS = `.ag-theme-${theme} .ag-row-selected {${this.jsxToCss(contextStyles.rowSelected)}} `;
        const cellCSS = `.ag-theme-${theme} .ag-cell {${this.jsxToCss(contextStyles.cell)}} `;
        const rowCSS = `.ag-row {${this.jsxToCss(contextStyles.row)}} `;
        const oddRowCSS = `.ag-row-odd {${this.jsxToCss(contextStyles.oddRows)}} `;
        const evenRowCSS = `.ag-row-even {${this.jsxToCss(contextStyles.evenRows)}} `;
        const headerCellCSS = `.ag-theme-${theme} .ag-header-cell {${this.jsxToCss(contextStyles.headerCell)}}`;
        const headerTextCSS = `.customHeaderLabel {${this.jsxToCss(contextStyles.headerText)}}`;
        const joinedCSS = `${rowCSS}${rowSelectedCSS}${cellCSS}${headerCellCSS}${headerTextCSS}${oddRowCSS}${evenRowCSS}`;

        const sheet = document.createElement('style');
        sheet.id = 'styleSheetForContextStyles';
        sheet.innerHTML = joinedCSS;

        // Remove previous instance if available.
        const styleSheet = document.getElementById(sheet.id);
        if (styleSheet) {
            const styleSheetParent = styleSheet.parentNode;
            styleSheetParent.removeChild(styleSheet);
        }

        // append updated style element to document
        document.body.appendChild(sheet);

        return (
            <div
                style={ contextStyles.container }
                className="gridContainer"
                data-testid={ testID && `${testID}__grid` }>
                <div
                    style={ {
                        height: contextStyles.container.padding ? `calc(${contextStyles.container.height} - 2 * ${contextStyles.container.padding})` : contextStyles.container.height,
                        maxHeight: contextStyles.container.padding ? `calc(100vh - 2 * ${contextStyles.container.padding})` : '100vh',
                    } }
                    className={ `ag-theme-${theme} gridViewport` }>
                    <AgGridReact

                        // listening for events
                        onRowDoubleClicked={ this.handleRowDoubleClicked }
                        onGridReady={ this.handleGridReady }
                        onColumnMoved={ this.handleColumnMoved }
                        onColumnPinned={ this.handleColumnPinnedStatusChange }
                        onColumnResized={ this.handleColumnResized }

                        // binding to array properties
                        rowData={ rows }

                        rowSelection={ multiSelectMode ? 'multiple' : 'single' }
                        suppressCellSelection

                        // setting default column properties
                        defaultColDef={ {
                            resizable: true,
                            sortable: true,
                            headerComponentFramework: SortableHeaderComponent,
                            columnStyles: { ...contextStyles.column },
                            columnList,
                            headerComponentParams: {
                                columnList,
                                handlePinClick: this.handlePinClick,
                                onSortRequested: this.handleSortRequested,
                                handleFilterToggle: this.handleFilterToggle,
                            },
                        } }>
                        {
                            this.renderColumn(columnList)
                        }
                    </AgGridReact>
                </div>
            </div>
        );
    }

    componentDidUpdate(prevProps, prevState) {
        const { cols } = this.props;

        // Update the columnList state when 'cols' prop is different
        if (cols !== prevState.columnList) {
            this.updateCols(cols);
        }

        const { rows } = this.props;
        if (rows !== prevProps.rows) {
            this.api.resetRowHeights();
        }
    }

    /**
     * Renders columns
     * @method renderColumn
     * @param {Object} columnList Object containing meta data about columns
     */
    renderColumn = (columnList) =>
        columnList.map((item) =>
            (
                <AgGridColumn
                    autoHeight
                    sortable={ item.sortable }
                    key={ item.field }
                    pinned={ item.pinned }
                    field={ item.field }
                    width={ item.width }
                    headerName={ item.headerName }
                    cellRendererFramework={ item.cellRenderer ? item.cellRenderer : defaultCellRenderer } />
            )
        )

    /** Render filter dialog into filterContainer div */
    renderFilterDialog = () => {
        const filterContainer = document.getElementById('filter');

        const { contextStyles } = this.props;

        const {
            columnList,
            colWidths,
            isFilterShowing,
        } = this.state;

        const unpinnedColsViewport = document.getElementsByClassName('ag-center-cols-viewport')[0];
        const viewPortScrollLeft = unpinnedColsViewport.scrollLeft;

        if (isFilterShowing && filterContainer) {
            // apply filterContainer styles.
            Object.keys(contextStyles.filterContainer).map((key) => {
                filterContainer.style[key] = contextStyles.filterContainer[key];
            });

            // scroll handler
            filterContainer.onscroll = this.handleFilterDialogScroll;
        }
        else if (filterContainer) {
            // Clear styles when filter is not showing
            filterContainer.style = {};
        }

        ReactDOM.render(
            <FilterComponent
                contextStyles={ contextStyles }
                onApplyFilters={ this.handleApplyFilters }
                left={ viewPortScrollLeft }
                columnList={ columnList }
                colWidths={ colWidths }
                onFilterToggle={ this.handleFilterToggle }
                isFilterShowing={ isFilterShowing }
                api={ this.api } />,
            filterContainer);
    }

    /**
    * Helper method to update cols
    * @param { array } cols - The new columnlist
    */
    updateCols = (cols) => {
        this.setState({ columnList: cols });
    };


    /**
     * Return key value pairs combined in a single string in format CSS expects (keys as dashed strings) for style object received
     * @method jsxToCss
     * @param {Object} styleObject Object containing style information in format JSX expects (camelcase)
     * @returns {String} css in string
     */
    jsxToCss = (styleObject) => {
        const keys = Object.keys(styleObject);
        let cssString = '';
        let dashedKey;
        keys.forEach((key) => {
            // Convert camelcase to dashed
            dashedKey = key.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();

            // Append to existing string
            cssString = `${cssString}${dashedKey}: ${styleObject[key]};`;
        });
        return cssString;
    }

    /**
     * Handles apply filter request.
     */
    handleApplyFilters = (filterData) => {
        const { onApplyFilters } = this.props;

        return new Promise((resolve, reject) => {
            if (onApplyFilters) {
                onApplyFilters(filterData)
                    .then(() => resolve())
                    .catch((err) => reject(err));
            }
            else {
                reject('No method passed to handle filtering.');
            }
        });
    }

    /**
     * Handles moving of column
     */
    handleColumnMoved = (event) => {
        if (event.finished) {
            const { columnList } = this.state;
            const {
                column,
                toIndex,
            } = event;
            let movedColumnIndex;

            // Get index of moved column
            columnList.map((col, index) => {
                if (col.field === column.colId) {
                    movedColumnIndex = index;
                }
            });

            const newColumnList = [ ...columnList ];
            const movedColumn = columnList[movedColumnIndex];
            newColumnList.splice(movedColumnIndex, 1);
            newColumnList.splice(toIndex, 0, movedColumn);

            this.setState({ columnList: newColumnList }, () => {
                this.renderFilterDialog();
            });
        }
    }

    /**
     * Makes grid api available to component and adds a div for filterContainer to DOM.
     */
    handleGridReady = (params) => {
        this.api = params.api;
        this.api.resetRowHeights();

        // Add a div to render filterDialogContainer for the grid
        const filterContainer = document.createElement('div');
        filterContainer.id = 'filter';
        const elements = document.getElementsByClassName('ag-full-width-container');
        const rootGridElement = elements[0];

        if (!rootGridElement.childNodes.length) {
            rootGridElement.appendChild(filterContainer);
        }
        else if (rootGridElement.childNodes[0].id !== 'filter') {
            rootGridElement.insertBefore(filterContainer, rootGridElement.childNodes[0]);
        }

        // Add event listener for scroll event on unpinned col viewport so as to scroll filterContainer accordingly.
        const unpinnedColsViewport = document.getElementsByClassName('ag-center-cols-viewport')[0];
        unpinnedColsViewport.addEventListener('scroll', () => {
            const filterContainer = document.getElementById('filter');

            this.setState({ viewPortScrollLeft: unpinnedColsViewport.scrollLeft }, () => {
                const { viewPortScrollLeft } = this.state;
                filterContainer.scrollLeft = viewPortScrollLeft;
                this.renderFilterDialog();
            });
        });
    };

    /**
     * Handles resize of columns
     * @method handleColumnResized
     * @param event column resize event
     */
    handleColumnResized = (event) => {
        if (event.finished) {
            // Recalculate row heights.
            this.api.resetRowHeights();

            const { colWidths } = this.state;
            colWidths.set(event.column.userProvidedColDef.field, event.column.actualWidth);

            // Render filterDialog with updated colWidths
            this.setState({ colWidths }, () => this.renderFilterDialog());
        }
    }

    /**
     * Handles double click on row(calls action received from props)
     * @method handleRowDoubleClicked
     * @param event RowEvent
     */
    handleRowDoubleClicked = (event) => {
        // Run action on row selection if available
        const { onRowDblClick } = this.props;

        if (onRowDblClick) {
            onRowDblClick(event.node.data);
        }
    };

    /**
     * Handles sorting request
     * @param {String} sortOrder direction of sorting, asc, or desc or ""
     */
    handleSortRequested = (order) => {
        const { onSortRequested } = this.props;
        return new Promise((resolve, reject) => {
            if (onSortRequested) {
                onSortRequested(order)
                    .then(() => resolve(order))
                    .catch((err) => reject(err));
            }
        });
    }

    /**
     * On column pin status change call updateColumnPinnedStatus on the column with changed value
     * @method handleColumnPinnedStatusChange
     * @param {Event} event grid columnPinned event
     */
    handleColumnPinnedStatusChange = (event) => {
        const { colId } = event.column;
        const { pinned } = event;
        const {
            columnList,
            isFilterShowing,
        } = this.state;
        columnList.forEach((column) => {
            // AG-Grid sometimes adds additional character to colId, slice removes them for comparison.
            if (column.field === colId || column.field === colId.slice(0, -2)) {
                column.pinned = !!pinned;
            }
        });

        this.setState({ columnList }, () => {
            if (isFilterShowing) {
                this.renderFilterDialog();
            }
        });
    }

    /**
     * handles click on pin icon
     * @method handlePinClick
     * @param { string }colHeader name of the column
     */
    handlePinClick = (colHeader) => {
        const {
            columnList,
            isFilterShowing,
        } = this.state;
        const newColumnList = [ ...columnList ];
        newColumnList.forEach((col) => {
            if (col.headerName === colHeader) {
                col.pinned = !col.pinned;
            }
        });

        this.setState({ columnList }, () => {
            if (isFilterShowing) {
                this.renderFilterDialog();
            }
        });
    }

    /**
     * Handles sorting
     */
    handleSortRequested = (order) => {
        const { onSortRequested } = this.props;
        return onSortRequested(order);
    }

    /**
     * Handles show/hide for filter
     * @method handleFilterToggle
     * @param { boolean }isFilterShowing
     */
    handleFilterToggle = (isFilterShowing) => {
        this.setState({ isFilterShowing }, () => this.renderFilterDialog());
    }

    /**
     * Synchronize filter scrolling with column scrolling
     * @method handleFilterDialogScroll
     * @param e scroll event
     */
    handleFilterDialogScroll = (e) => {
        const unpinnedColsViewport = document.getElementsByClassName('ag-center-cols-viewport')[0];
        unpinnedColsViewport.scrollLeft = e.target.scrollLeft;
    }
}

DataGrid.propTypes = {
    /** action on cell selection */
    onRowDblClick: PropTypes.func,

    /** testid */
    testID: PropTypes.string,

    /** Array of records */
    rows: PropTypes.array.isRequired,

    /** Array containing column defintion object */
    cols: PropTypes.arrayOf(PropTypes.shape({
        /** field(unique name for col) */
        field: PropTypes.string,

        /** headerName(column header name) */
        headerName: PropTypes.string,

        /** Width of column */
        width: PropTypes.number,

        /** If column is pinned */
        pinned: PropTypes.bool,

        /** If column is sortable */
        sortable: PropTypes.bool,

        /**
         * Cell Renderer
         * @param {Object} params column details for the cell
         */
        cellRenderer: PropTypes.func,
    })).isRequired,

    /** If multiple row selection is enabled */
    multiSelectMode: PropTypes.bool,

    /** Handle Sorting and return sort order, 'asc', 'desc', or '' for no ordering */
    onSortRequested: PropTypes.func,

    /** Apply filters */
    onApplyFilters: PropTypes.func,

    /** contextStyles object for grid */
    contextStyles: PropTypes.shape({
        /** Style object for container */
        container: PropTypes.object,

        /** Style object for grid */
        grid: PropTypes.object,

        /** Style object for header cell */
        headerCell: PropTypes.object,

        /** Style object for header text */
        headerText: PropTypes.object,

        /** Style object for grid row */
        row: PropTypes.object,

        /** Style object for selected grid row */
        rowSelected: PropTypes.object,

        /** Style object for oddRows*/
        oddRows: PropTypes.object,

        /** Style object for evenRows*/
        evenRows: PropTypes.object,

        /** Style object for grid filter dialog */
        filter: PropTypes.object,

        /** Style object for grid filter dialog container */
        filterContainer: PropTypes.object,

        /** Column level styles */
        column: PropTypes.object,
    }),
};

const defaultCellRenderer = (props) => {
    // colDef(column definition containing info about current column) and data(one row or record) are passed by ag-grid
    const {
        // eslint-disable-next-line react/prop-types
        colDef,
        // eslint-disable-next-line react/prop-types
        data,
    } = props;

    // Find property corresponding to field in columnDef.
    const cell = data.properties.filter((col) => col.name === colDef.field)[0];
    return (
        <div style={ {
            paddingLeft: '15px',
        } }>
            { ( typeof cell.value === 'string' || typeof cell.value === 'number' ) ? cell.value : '' }
            { ( typeof cell.value === 'object' && cell.value ) ? cell.value.code : '' }
        </div>
    );
};
